//
//  Copyright (C) 2004-2006  Autodesk, Inc.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of version 2.1 of the GNU Lesser
//  General Public License as published by the Free Software Foundation.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

#ifndef POINTSYMBOLIZATION2D_H_
#define POINTSYMBOLIZATION2D_H_

#include "Symbolization.h"
#include "Symbol.h"

BEGIN_NAMESPACE_MDFMODEL

    //-------------------------------------------------------------------------
    // DESCRIPTION:
    // The PointSymbolization2D class defines a way to specify the stylization
    // details for features that contain point geometric primitives.
    //-------------------------------------------------------------------------
    class MDFMODEL_API PointSymbolization2D : public Symbolization
    {
    public:
        // Construction, destruction, initialization
        PointSymbolization2D();
        virtual ~PointSymbolization2D();

        //Operations
        //Property : Symbol
        Symbol* GetSymbol();
        void AdoptSymbol(Symbol* psymAdoptedSymbol);
        Symbol* OrphanSymbol();

        // Visitor Pattern method defined in Symbol.
        virtual void AcceptVisitor(ISymbolizationVisitor& isyVisitor);

    private:
        // Hidden copy constructor and assignment operator.
        PointSymbolization2D(const PointSymbolization2D&);
        PointSymbolization2D& operator=(const PointSymbolization2D&);

        // Data members
        // Defines the appearance of the point.
        Symbol* m_psymSymbol;

    };

END_NAMESPACE_MDFMODEL
#endif //POINTSYMBOLIZATION2D_H_
